/* ============================================================================
 * Copyright (c) 2010, Michael A. Jackson (BlueQuartz Software)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice, this
 * list of conditions and the following disclaimer in the documentation and/or
 * other materials provided with the distribution.
 *
 * Neither the name of Michael A. Jackson nor the names of its contributors may
 * be used to endorse or promote products derived from this software without
 * specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 //
 //  This code was written under United States Air Force Contract number
 //                           FA8650-07-D-5800
 //
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

#include <iostream>
#include <map>
#include <string>
#include <sstream>

#include "MXA/Common/LogTime.h"

#include "ZeissXML/ZeissXMLReader.h"
#include "ZeissXML/ZeissTagMapping.h"
#include "ZeissXML/ZeissMetaFactory.h"
#include "ZeissXML/ZeissMetaEntry.h"
#include "ZeissXML/ZeissDirectoryPatterns.h"


#include "tclap/CmdLine.h"
#include "tclap/ValueArg.h"

// -----------------------------------------------------------------------------
//
// -----------------------------------------------------------------------------
void printData(ZeissTagsXmlSection::MetaDataPtrType metaDataMap, int idTag)
{
  ZeissTagMapping::Pointer tagMapping = ZeissTagMapping::instance();
  ZeissTagsXmlSection::MetaDataType::const_iterator e = metaDataMap->find(idTag);
  if (e != metaDataMap->end())
  {
   // std::cout << tagMapping->nameForId(idTag) << ": ";
    (*e).second->printValue(std::cout);
   // std::cout << std::endl;
  }
}


/**
* @brief This program will read in Zeiss XML files and look for specific tags
* related to the "focus" meta data. This main program is just a one-off program
* but the embedded ZeissLibrary should be reusable in other projects where
* a developer needs to read the Zeiss XML files.
*/
int main(int argc, char **argv) {
//  std::cout << logTime() << "Starting ParseXML" << std::endl;

  TCLAP::CmdLine cmd("", ' ', "1.0.0");

  TCLAP::ValueArg<std::string> in_prefix("", "prefix", "", false, "", "");
  cmd.add(in_prefix);
  TCLAP::ValueArg<std::string> in_suffix("", "suffix", "", false, "", "");
  cmd.add(in_suffix);
  TCLAP::ValueArg<std::string> in_xml("", "xml", "", true, "", "");
  cmd.add(in_xml);
  try
  {
    int error = 0;
    cmd.parse(argc, argv);

    std::string curPrefix = "C";
    std::string curSuffix = in_suffix.getValue();
    std::string prefix = in_prefix.getValue();
    std::stringstream ss;
    //std::cout << "File\t" << Zeiss::MetaXML::FocusPosition << "\t" << Zeiss::MetaXML::FocusCalibrated << "\t" << Zeiss::MetaXML::FocusDepth << std::endl;



    ZeissXMLReader::Pointer reader = ZeissXMLReader::New(in_xml.getValue());
    int err = reader->parse();
    if(err < 0)
    {
      std::cout << "Error reading the XML File '" << ss.str() << "'" << std::endl;
      return EXIT_FAILURE;
    }

    ZeissTagsXmlSection::Pointer tags = reader->getTagsSection();
    ZeissTagsXmlSection::MetaDataPtrType metaDataMap = tags->getMetaDataMap();
    ZeissTagMapping::Pointer tagMapping = ZeissTagMapping::instance();
    ZeissTagsXmlSection::MetaDataType::const_iterator e;
#if 0
    for (ZeissTagsXmlSection::MetaDataType::iterator iter = metaDataMap->begin(); iter != metaDataMap->end(); ++iter )
    {
      std::cout << (*iter).first << " ";
      //(*iter).second->printValue(std::cout);

      e = metaDataMap->find((*iter).first );
      if (e != metaDataMap->end())
      {
        std::cout << tagMapping->nameForId((*iter).first ) << ": ";
        (*e).second->printValue(std::cout);
        // std::cout << std::endl;
      }
      std::cout << std::endl;
    }
#endif

    e = metaDataMap->find(Zeiss::MetaXML::ScaleFactorForWidthId);
    if (e != metaDataMap->end())
    {
      std::cout << "Scale For Width: ";
      (*e).second->printValue(std::cout);
      std::cout << std::endl;
    }

    e = metaDataMap->find(Zeiss::MetaXML::ScaleFactorForHeightId);
    if (e != metaDataMap->end())
    {
      std::cout << "Scale For Height: ";
      (*e).second->printValue(std::cout);
      std::cout << std::endl;
    }

    e = metaDataMap->find(Zeiss::MetaXML::CameraFrameWidthId);
    if(e != metaDataMap->end())
    {
      std::cout << "Image Width (Pixels): ";
      (*e).second->printValue(std::cout);
      std::cout << std::endl;
    }
    e = metaDataMap->find(Zeiss::MetaXML::CameraFrameHeightId);
    if(e != metaDataMap->end())
    {
      std::cout << "Image Height (Pixels): ";
      (*e).second->printValue(std::cout);
      std::cout << std::endl;
    }


    // Find the number of images and loop over the meta data for each image extracting
    // out the stage coordinates for the origin of each image.
    std::vector<ZeissTagsXmlSection::Pointer> imageTags = reader->getImageTags();
    std::cout << "Tile_Number,X_Origin,Y_Origin" << std::endl;

    for(int i = 0; i < imageTags.size(); ++i)
    {
      metaDataMap = imageTags[i]->getMetaDataMap();
      std::cout << i << ",";
      printData(metaDataMap, Zeiss::MetaXML::ImageOrgCoordinateUId);
      std::cout << ",";
      printData(metaDataMap, Zeiss::MetaXML::ImageOrgCoordinateVId);
      std::cout << std::endl;
    }


#if 0
    std::cout << "File\t" << Zeiss::MetaXML::FocusPosition << "\t" << Zeiss::MetaXML::FocusCalibrated << "\t" << Zeiss::MetaXML::FocusDepth << std::endl;
    for (int i = 1; i < 6; ++i)
    {
      for (int j = 1; j < 21; ++j)
      {
        ss.str("");
        ss << prefix << curPrefix << i << "_" << j << curSuffix;

        ZeissXMLReader::Pointer reader = ZeissXMLReader::New(ss.str());
        int err = reader->parse();
        if (err < 0)
        {
          std::cout << "Error reading the XML File '" << ss.str() << "'" << std::endl;
          return EXIT_FAILURE;
        }

        ZeissTagsXmlSection::Pointer tags = reader->getTagsSection();
        ZeissTagsXmlSection::MetaDataPtrType metaDataMap = tags->getMetaDataMap();
        std::cout << curPrefix << i << "_" << j << curSuffix << "\t";
        printData(metaDataMap, Zeiss::MetaXML::FocusPositionId);
        std::cout << "\t";
        printData(metaDataMap, Zeiss::MetaXML::FocusCalibratedId);
        std::cout << "\t";
        printData(metaDataMap, Zeiss::MetaXML::FocusDepthId);
        std::cout << std::endl;
      }
    }
#endif

  }
  catch (TCLAP::ArgException &e)
  {
    std::cerr << " error: " << e.error() << " for arg " << e.argId() << std::endl;
    std::cout << "** Unknown Arguments. Displaying help listing instead. **" << std::endl;
    return -1;
  }

 // std::cout << "-----------------------------------------------------------" << std::endl;

  return EXIT_SUCCESS;
}
