///////////////////////////////////////////////////////////////////////////////
//
//  Copyright (c) 2007, 2010 Michael A. Jackson for BlueQuartz Software
//  All rights reserved.
//  BSD License: http://www.opensource.org/licenses/bsd-license.html
//
//  This code was written under United States Air Force Contract number
//                           F33615-03-C-5220
//
///////////////////////////////////////////////////////////////////////////////
#ifndef ZEISSTAGMAPPING_H_
#define ZEISSTAGMAPPING_H_


#include <ZeissXML/ZeissMetaFactory.h>

#include <MXA/MXATypes.h>
#include <MXA/Common/MXASetGetMacros.h>
#include <MXA/Common/LogTime.h>

#include <map>

typedef std::map<int, std::string>  ZeissIdNameMap;
typedef std::map<std::string, int>  ZeissNameIdMap;
typedef std::map<int, ZeissMetaFactory::Pointer> ZeissFactoryMap;

/**
* @class ZeissTagMapping ZeissTagMapping.h R3D/Common/ZeissTagMapping.h
* @brief This class is used to map values from the Zeiss XML file to and from
* human readable values. It also creates factories for each of the tags found in the
* Zeiss XML file. For each value there is a const std::string and an integer value
* that uniquely identifies the tag.
* @author Michael A. Jackson for BlueQuartz Software
* @date Jul 29, 2009
* @version 1.0
*/
class ZeissTagMapping
{
  public:
    MXA_SHARED_POINTERS(ZeissTagMapping);

    virtual ~ZeissTagMapping();

    static ZeissTagMapping::Pointer instance();


    std::string nameForId(int idtag);
    int idForName(const std::string &name);
    ZeissMetaFactory::Pointer factoryForId(int idTag);
    AbstractZeissMetaData::Pointer metaDataForId(int idTag, const std::string &value);
    AbstractZeissMetaData::Pointer metaDataForTagName(const std::string &name);

  protected:
    ZeissTagMapping();

    void initIdNameMap();
    void initNameIdMap();
    void initFactoryMap();

  private:
    ZeissIdNameMap _idNameMap;
    ZeissNameIdMap _nameIdMap;
    ZeissFactoryMap _idFactoryMap;


    ZeissTagMapping(const ZeissTagMapping&);    // Copy Constructor Not Implemented
    void operator=(const ZeissTagMapping&);  // Operator '=' Not Implemented

};

namespace Zeiss
{
namespace MetaXML
{

static const int ImageDataMappingTableId = 257;
static const int ImageDataMappingLowId = 258;
static const int ImageDataMappingHighId = 259;
static const int ImageDataMappingAutoRangeId = 260;
static const int DocumentThumbnailId = 261;
static const int ImageDataMappingGammaId = 262;
static const int ImageDataHistoMaxId = 263;
static const int ImageOverExposureId = 264;
static const int ImageRelativeTimeFirstId = 265;
static const int ImageRelativeTime1Id = 265;
static const int ImageRelativeTime2Id = 266;
static const int ImageRelativeTime3Id = 267;
static const int ImageRelativeTime4Id = 268;
static const int ImageRelativeTimeId = 300;
static const int ImageBaseTimeFirstId = 301;
static const int ImageBaseTime1Id = 301;
static const int ImageBaseTime2Id = 302;
static const int ImageBaseTime3Id = 303;
static const int ImageBaseTime4Id = 304;
static const int ImageBaseTimeLast_reservedId = 332;
static const int ImageObjectTypeId = 513;
static const int ImageWidthId = 515;
static const int ImageHeightId = 516;
static const int ImageCountRawId = 517;
static const int ImagePixelTypeId = 518;
static const int CountId = 519;
static const int CompressionFactorForSaveId = 521;
static const int DocumentSaveFlagsId = 522;
static const int ImageTimestampCommentsId = 523;
static const int DocumentSubTypeId = 530;
static const int CameraImageValidBitsPerPixelId = 531;
static const int ImageMemoryUsageId = 532;
static const int ImageMSBShiftForPixelsId = 533;
static const int ScaleFactorForWidthId = 769;
static const int ScaleTypeForWidthId = 770;
static const int ScaleWidthId = 771;
static const int ScaleFactorForHeightId = 772;
static const int ScaleTypeForHeightId = 773;
static const int ScaleHeightId = 774;
static const int ScaleFactorForDepthId = 775;
static const int ScaleTypeForDepthId = 776;
static const int ScaleDepthId = 777;
static const int ScalingParentId = 778;
static const int EventLogTypeId = 1000;
static const int EventLogDateId = 1001;
static const int EventLogCodeId = 1002;
static const int EventLogSourceNameId = 1003;
static const int EventLogMessageId = 1004;
static const int AcquisitionGroupId = 1024;
static const int CameraImageAcquisitionTimeId = 1025;
static const int CameraValidBitsPerPixelId = 1027;
static const int MonoReferenceLowId = 1029;
static const int MonoReferenceHighId = 1030;
static const int RedReferenceLowId = 1031;
static const int RedReferenceHighId = 1032;
static const int GreenReferenceLowId = 1033;
static const int GreenReferenceHighId = 1034;
static const int BlueReferenceLowId = 1035;
static const int BlueReferenceHighId = 1036;
static const int GrabberNameId = 1041;
static const int CameraNameId = 1042;
static const int CameraTriggerSignalTypeId = 1044;
static const int CameraTriggerEnableId = 1045;
static const int GrabberTimeoutId = 1046;
static const int AcquisitionTimeInDoubleId = 1047;
static const int MultiChannelEnabledId = 1281;
static const int MultiChannelColorId = 1282;
static const int MultiChannelWeightId = 1283;
static const int MultiChannelNameId = 1284;
static const int DocumentInformationGroupId = 1536;
static const int DocumentTitleId = 1537;
static const int DocumentAuthorId = 1538;
static const int DocumentKeywordsId = 1539;
static const int DocumentCommentsId = 1540;
static const int DocumentSampleIDId = 1541;
static const int DocumentSubjectId = 1542;
static const int RevisionNumberId = 1543;
static const int DocumentPathId = 1544;
static const int FileLinkId = 1545;
static const int DocumentTypeId = 1546;
static const int FileMediumId = 1547;
static const int FileIDId = 1548;
static const int DocumentFileNameId = 1549;
static const int FileDateId = 1550;
static const int FileSizeId = 1551;
static const int DocumentNameId = 1553;
static const int FileAttributesId = 1554;
static const int ProjectGroupId = 1792;
static const int DateId = 1793;
static const int DocumentLastSavedById = 1794;
static const int UserCompanyId = 1795;
static const int UserCompanyLogoId = 1796;
static const int ImageId = 1797;
static const int UserIdId = 1800;
static const int UserNameId = 1801;
static const int UserCityId = 1802;
static const int UserAddressId = 1803;
static const int UserStateId = 1804;
static const int UserPhoneId = 1805;
static const int UserFaxId = 1806;
static const int ObjectiveId = 2049;
static const int OptovarId = 2050;
static const int ReflectorId = 2051;
static const int CondenserContrastId = 2052;
static const int TransmissionFilter1Id = 2053;
static const int TransmittedLightFilter1Id = 2053;
static const int TransmissionFilter2Id = 2054;
static const int TransmittedLightFilter2Id = 2054;
static const int InternalShutterId = 2055;
static const int ReflectedLightShutterId = 2055;
static const int CondenserFrontlensId = 2056;
static const int ExcitationFilterId = 2057;
static const int TransmittedLightFieldstopId = 2060;
static const int ReflectedLightApertureId = 2061;
static const int CondenserNAId = 2062;
static const int LightPathId = 2063;
static const int HalogenLampOnId = 2064;
static const int HalogenLampModeId = 2065;
static const int HalogenLampVoltageId = 2066;
static const int FluoLampModeId = 2068;
static const int FluoLampIntensityId = 2069;
static const int LightmanagerEnabledId = 2070;

static const int FocusPositionId = 2072;
static const int ImageCoordinateUId = 2073;
//static const int StageXPositionId = 2073;
static const int ImageCoordinateVId = 2074;
//static const int StageYPositionId = 2074;
static const int MicroscopeNameId = 2075;
static const int ObjectiveMagnificationId = 2076;
static const int ObjectiveNAId = 2077;
static const int MicroscopeIlluminationId = 2078;
static const int ExtraShutter1Id = 2079;
static const int ExtraShutter2Id = 2080;
static const int ExtraShutter3Id = 2081;
static const int ExtraFilter1Id = 2082;
static const int ExtraFilter2Id = 2083;
static const int ParfocalCorrectionId = 2084;
static const int ExtraShutter4Id = 2086;
static const int ExtraShutter5Id = 2087;
static const int ExtraShutter6Id = 2088;
static const int ExtraFilter3Id = 2089;
static const int ExtraFilter4Id = 2090;
static const int ObjectivePositionId = 2103;
static const int ObjectiveContrastMethodId = 2104;
static const int ObjectiveImmersionTypeId = 2105;
static const int ReflectorPositionId = 2107;
static const int TransmissionFilter1PositionId = 2109;
static const int TransmittedLightFilter1PositionId = 2109;
static const int TransmissionFilter2PositionId = 2110;
static const int TransmittedLightFilter2PositionId = 2110;
static const int ExcitationFilterPositionId = 2112;
static const int LampMirrorPositionId = 2113;
static const int ExtraFilter1PositionId = 2114;
static const int ExtraFilter2PositionId = 2115;
static const int ExtraFilter3PositionId = 2116;
static const int ExtraFilter4PositionId = 2117;
static const int LightmanagerModeId = 2118;
static const int HalogenLampCalibratedId = 2119;
static const int CondenserNAGoSpeedId = 2120;
static const int TransmittedLightFieldstopGoSpeedId = 2121;
static const int OptovarGoSpeedId = 2122;
static const int FocusCalibratedId = 2123;
static const int FocusBasicPositionId = 2124;
static const int FocusPowerId = 2125;
static const int FocusBacklashId = 2126;
static const int FocusMeasurementOriginId = 2127;
static const int FocusMeasurementDistanceId = 2128;
static const int FocusSpeedId = 2129;
static const int FocusGoSpeedId = 2130;
static const int FocusStepDistanceId = 2131;
static const int FocusInitPositionId = 2132;
static const int StageCalibratedId = 2133;
static const int StagePowerId = 2134;
static const int StageXBacklashId = 2135;
static const int StageYBacklashId = 2136;
static const int StageXSpeedId = 2137;
static const int StageYSpeedId = 2138;
static const int StageSpeedId = 2139;
static const int StageXGoSpeedId = 2140;
static const int StageYGoSpeedId = 2141;
static const int StageXStepDistanceId = 2142;
static const int StageYStepDistanceId = 2143;
static const int StageXInitPositionId = 2144;
static const int StageYInitPositionId = 2145;
static const int MicroscopeMagnificationId = 2146;
static const int ReflectorMagnificationId = 2147;
static const int LampMirrorSelectionId = 2148;
static const int FocusDepthId = 2149;
static const int MicroscopeTypeId = 2150;
static const int ObjectiveWorkingDistanceId = 2151;
static const int ReflectedLightApertureGoSpeedId = 2152;
static const int ExternalShutterId = 2153;
static const int ObjectiveImmersionStopActiveId = 2154;
static const int FocusStartSpeedId = 2155;
static const int FocusAccelerationId = 2156;
static const int ReflectedLightFieldstopId = 2157;
static const int ReflectedLightFieldstopGoSpeedId = 2158;
static const int ReflectedLightFilter1Id = 2159;
static const int ReflectedLightFilter2Id = 2160;
static const int ReflectedLightFilter1PositionId = 2161;
static const int ReflectedLightFilter2PositionId = 2162;
static const int TransmittedLightAttenuatorId = 2163;
static const int ReflectedLightAttenuatorId = 2164;
static const int TransmittedLightShutterId = 2165;
static const int TransmittedLightAttenuatorGoSpeedId = 2166;
static const int ReflectedLightAttenuatorGoSpeedId = 2167;
static const int ApotomeFocusPositionId = 2168;
static const int ApotomeFocusGoSpeedId = 2169;
static const int ApotomePhasePositionId = 2170;
static const int ApotomeSliderPositionId = 2171;
static const int ApotomeGridLinesPerMillimeterId = 2172;
static const int ApotomeWobbleMinPositionId = 2173;
static const int ApotomeWobbleMaxPositionId = 2174;
static const int ApotomeWobbleSpeedId = 2175;
static const int TransmittedLightVirtualFilterPositionId = 2176;
static const int TransmittedLightVirtualFilterId = 2177;
static const int ReflectedLightVirtualFilterPositionId = 2178;
static const int ReflectedLightVirtualFilterId = 2179;
static const int ReflectedLightHALModeId = 2180;
static const int ReflectedLightHALVoltageId = 2181;
static const int ReflectedLightHALColourTemperatureId = 2182;
static const int ContrastmanagerModeId = 2183;
static const int DazzleProtectionId = 2184;
static const int StageXCalibratedId = 2185;
static const int StageYCalibratedId = 2186;
static const int StageXPowerId = 2187;
static const int StageYPowerId = 2188;
static const int StageXStartSpeedId = 2189;
static const int StageYStartSpeedId = 2190;
static const int StageXAccelerationId = 2191;
static const int StageYAccelerationId = 2192;
static const int StageStartSpeedId = 2193;
static const int StageAccelerationId = 2194;
static const int ZoomId = 2195;
static const int ZoomGoSpeedId = 2196;
static const int LightZoomId = 2197;
static const int LightZoomGoSpeedId = 2198;
static const int LightZoomCoupledId = 2199;
static const int TransmittedLightHALModeId = 2200;
static const int TransmittedLightHALVoltageId = 2201;
static const int TransmittedLightHALColourTemperatureId = 2202;
static const int ReflectedLightCLModeId = 2203;
static const int ReflectedLightCLIntensityId = 2204;
static const int ReflectedLightCLColourTemperatureId = 2205;
static const int TransmittedLightCLModeId = 2206;
static const int TransmittedLightCLIntensityId = 2207;
static const int TransmittedLightCLColourTemperatureId = 2208;
static const int InfinitySpaceChangerPositionId = 2209;
static const int SidePortPercentId = 2210;
static const int TwoTvVisCamChangerPositionId = 2211;
static const int OcularPercentId = 2212;
static const int TwoTvCamerasChangerPositionId = 2213;
static const int TopPortPercentId = 2214;
static const int OcularShutterId = 2215;
static const int TwoTvCamerasChangerCubeId = 2216;
static const int LightWaveLengthId = 2217;
static const int OcularMagnificationId = 2218;
static const int CameraAdapterMagnificationId = 2219;
static const int MicroscopePortId = 2220;
static const int OcularTotalMagnificationId = 2221;
static const int FieldOfViewId = 2222;
static const int OcularId = 2223;
static const int CameraAdapterId = 2224;
static const int StageJoystickEnabledId = 2225;
static const int ContrastmanagerContrastMethodId = 2226;
static const int StageXJoystickEnabledId = 2227;
static const int StageYJoystickEnabledId = 2228;

static const int CameraFrameLeftId = 2307;
static const int CameraFrameTopId = 2308;
static const int CameraFrameWidthId = 2309;
static const int CameraFrameHeightId = 2310;
static const int CameraFrameBinningId = 2311;
static const int CameraFrameFullId = 2312;
static const int CameraFramePixelDistanceId = 2313;
static const int DataFormatUseScalingId = 2318;
static const int CameraFrameImageOrientationId = 2319;
static const int MeteorChannelInputId = 2322;
static const int MeteorChannelSyncId = 2323;
static const int CameraWhiteBalanceRedId = 2325;
static const int CameraWhiteBalanceGreenId = 2326;
static const int CameraWhiteBalanceBlueId = 2327;
static const int CameraFrameScalingFactorId = 2331;
static const int MeteorCameraTypeId = 2562;
static const int CameraExposureTimeValueId = 2564;
static const int CameraExposureTimeAutoCalculateId = 2568;
static const int MeteorGainValueId = 2569;
static const int MeteorGainAutomaticId = 2571;
static const int MeteorAdjustHueId = 2572;
static const int MeteorAdjustSaturationId = 2573;
static const int MeteorAdjustRedLowId = 2574;
static const int MeteorAdjustGreenLowId = 2575;
static const int MeteorAdjustBlueLowId = 2576;
static const int MeteorAdjustRedHighId = 2577;
static const int MeteorAdjustGreenHighId = 2578;
static const int MeteorAdjustBlueHighId = 2579;
static const int MeteorAdjustWhiteRefEnableId = 2580;
static const int CameraExposureTimeCalculationControlId = 2582;
static const int AxioCamFadingCorrectionEnableId = 2585;
static const int AxioCamSuspendLiveTimeoutId = 2586;
static const int CameraLiveImageId = 2587;
static const int CameraLiveEnabledId = 2588;
static const int LiveImageSyncObjectNameId = 2589;
static const int CameraLiveSpeedId = 2590;
static const int CameraImageId = 2591;
static const int CameraImageWidthId = 2592;
static const int CameraImageHeightId = 2593;
static const int CameraImagePixelTypeId = 2594;
static const int CameraImageShMemoryNameId = 2595;
static const int CameraLiveImageWidthId = 2596;
static const int CameraLiveImageHeightId = 2597;
static const int CameraLiveImagePixelTypeId = 2598;
static const int CameraLiveImageShMemoryNameId = 2599;
static const int CameraLiveMaximumSpeedId = 2600;
static const int CameraLiveBinningId = 2601;
static const int CameraLiveGainValueId = 2602;
static const int CameraLiveExposureTimeValueId = 2603;
static const int CameraLiveScalingFactorId = 2604;
static const int CameraCoAcqFastModeId = 2605;
static const int ImageIndexUId = 2817;
static const int ImageIndexVId = 2818;
static const int ImageIndexZId = 2819;
static const int ImageIndexCId = 2820;
static const int ImageIndexTId = 2821;
static const int ImageIndexPId = 2822;
static const int ImageIndexAId = 2823;
static const int ImageCountPId = 2824;
static const int ImageCountAId = 2825;
static const int ImageIndexSId = 2827;
static const int ImageIndexRawId = 2828;
static const int ImageCountZId = 2832;
static const int ImageCountCId = 2833;
static const int ImageCountTId = 2834;
static const int ImageCountUId = 2838;
static const int ImageCountVId = 2839;
static const int ImageCountSId = 2840;
static const int ImageOrgCoordinateUId = 2841;
static const int ImageOrgCoordinateVId = 2842;
static const int LayerDrawFlagsId = 3088;
static const int JobGroupId = 3328;
static const int JobNameId = 3329;
static const int JobInfoId = 3330;
static const int JobProgressId = 3331;
static const int JobStatusId = 3332;
static const int JobIdId = 3333;
static const int JobRemainingTimeId = 3334;
static const int RelFocusPosition1Id = 333;
static const int RelFocusPosition2Id = 334;
static const int RelFocusPositionId = 2071;
static const int SidePortTurretpositionId = 2252;
static const int SidePortLeftPercentId = 2253;
static const int SidePortRightPercentId = 2254;
static const int BasePortSliderPortPercentId = 2255;
static const int SidePortLeftFunctionId = 2256;
static const int SidePortRightFunctionId = 2257;
static const int BasePortSliderPortFunctionId = 2258;
static const int SidePortLightPathId = 2259;
static const int SidePortLightPathIDId = 2260;
static const int ObjectiveIDId = 2261;
static const int ReflectorIDId = 2262;

const std::string ImageDataMappingTable ("ImageDataMappingTable");
const std::string ImageDataMappingLow ("ImageDataMappingLow");
const std::string ImageDataMappingHigh ("ImageDataMappingHigh");
const std::string ImageDataMappingAutoRange ("ImageDataMappingAutoRange");
const std::string DocumentThumbnail ("DocumentThumbnail");
const std::string ImageDataMappingGamma ("ImageDataMappingGamma");
const std::string ImageDataHistoMax ("ImageDataHistoMax");
const std::string ImageOverExposure ("ImageOverExposure");
const std::string ImageRelativeTimeFirst ("ImageRelativeTimeFirst");
const std::string ImageRelativeTime1 ("ImageRelativeTime1");
const std::string ImageRelativeTime2 ("ImageRelativeTime2");
const std::string ImageRelativeTime3 ("ImageRelativeTime3");
const std::string ImageRelativeTime4 ("ImageRelativeTime4");
const std::string ImageRelativeTime ("ImageRelativeTime");
const std::string ImageBaseTimeFirst ("ImageBaseTimeFirst");
const std::string ImageBaseTime1 ("ImageBaseTime1");
const std::string ImageBaseTime2 ("ImageBaseTime2");
const std::string ImageBaseTime3 ("ImageBaseTime3");
const std::string ImageBaseTime4 ("ImageBaseTime4");
const std::string ImageBaseTimeLast_reserved ("ImageBaseTimeLast_reserved");
const std::string ImageObjectType ("ImageObjectType");
const std::string ImageWidth ("ImageWidth");
const std::string ImageHeight ("ImageHeight");
const std::string ImageCountRaw ("ImageCountRaw");
const std::string ImagePixelType ("ImagePixelType");
const std::string Count ("Count");
const std::string CompressionFactorForSave ("CompressionFactorForSave");
const std::string DocumentSaveFlags ("DocumentSaveFlags");
const std::string ImageTimestampComments ("ImageTimestampComments");
const std::string DocumentSubType ("DocumentSubType");
const std::string CameraImageValidBitsPerPixel ("CameraImageValidBitsPerPixel");
const std::string ImageMemoryUsage ("ImageMemoryUsage");
const std::string ImageMSBShiftForPixels ("ImageMSBShiftForPixels");
const std::string ScaleFactorForWidth ("ScaleFactorForWidth");
const std::string ScaleTypeForWidth ("ScaleTypeForWidth");
const std::string ScaleWidth ("ScaleWidth");
const std::string ScaleFactorForHeight ("ScaleFactorForHeight");
const std::string ScaleTypeForHeight ("ScaleTypeForHeight");
const std::string ScaleHeight ("ScaleHeight");
const std::string ScaleFactorForDepth ("ScaleFactorForDepth");
const std::string ScaleTypeForDepth ("ScaleTypeForDepth");
const std::string ScaleDepth ("ScaleDepth");
const std::string ScalingParent ("ScalingParent");
const std::string EventLogType ("EventLogType");
const std::string EventLogDate ("EventLogDate");
const std::string EventLogCode ("EventLogCode");
const std::string EventLogSourceName ("EventLogSourceName");
const std::string EventLogMessage ("EventLogMessage");
const std::string AcquisitionGroup ("AcquisitionGroup");
const std::string CameraImageAcquisitionTime ("CameraImageAcquisitionTime");
const std::string CameraValidBitsPerPixel ("CameraValidBitsPerPixel");
const std::string MonoReferenceLow ("MonoReferenceLow");
const std::string MonoReferenceHigh ("MonoReferenceHigh");
const std::string RedReferenceLow ("RedReferenceLow");
const std::string RedReferenceHigh ("RedReferenceHigh");
const std::string GreenReferenceLow ("GreenReferenceLow");
const std::string GreenReferenceHigh ("GreenReferenceHigh");
const std::string BlueReferenceLow ("BlueReferenceLow");
const std::string BlueReferenceHigh ("BlueReferenceHigh");
const std::string GrabberName ("GrabberName");
const std::string CameraName ("CameraName");
const std::string CameraTriggerSignalType ("CameraTriggerSignalType");
const std::string CameraTriggerEnable ("CameraTriggerEnable");
const std::string GrabberTimeout ("GrabberTimeout");
const std::string AcquisitionTimeInDouble ("AcquisitionTimeInDouble");
const std::string MultiChannelEnabled ("MultiChannelEnabled");
const std::string MultiChannelColor ("MultiChannelColor");
const std::string MultiChannelWeight ("MultiChannelWeight");
const std::string MultiChannelName ("MultiChannelName");
const std::string DocumentInformationGroup ("DocumentInformationGroup");
const std::string DocumentTitle ("DocumentTitle");
const std::string DocumentAuthor ("DocumentAuthor");
const std::string DocumentKeywords ("DocumentKeywords");
const std::string DocumentComments ("DocumentComments");
const std::string DocumentSampleID ("DocumentSampleID");
const std::string DocumentSubject ("DocumentSubject");
const std::string RevisionNumber ("RevisionNumber");
const std::string DocumentPath ("DocumentPath");
const std::string FileLink ("FileLink");
const std::string DocumentType ("DocumentType");
const std::string FileMedium ("FileMedium");
const std::string FileID ("FileID");
const std::string DocumentFileName ("DocumentFileName");
const std::string FileDate ("FileDate");
const std::string FileSize ("FileSize");
const std::string DocumentName ("DocumentName");
const std::string FileAttributes ("FileAttributes");
const std::string ProjectGroup ("ProjectGroup");
const std::string Date ("Date");
const std::string DocumentLastSavedBy ("DocumentLastSavedBy");
const std::string UserCompany ("UserCompany");
const std::string UserCompanyLogo ("UserCompanyLogo");
const std::string Image ("Image");
const std::string UserId ("UserId");
const std::string UserName ("UserName");
const std::string UserCity ("UserCity");
const std::string UserAddress ("UserAddress");
const std::string UserState ("UserState");
const std::string UserPhone ("UserPhone");
const std::string UserFax ("UserFax");
const std::string Objective ("Objective");
const std::string Optovar ("Optovar");
const std::string Reflector ("Reflector");
const std::string CondenserContrast ("CondenserContrast");
const std::string TransmissionFilter1 ("TransmissionFilter1");
const std::string TransmittedLightFilter1 ("TransmittedLightFilter1");
const std::string TransmissionFilter2 ("TransmissionFilter2");
const std::string TransmittedLightFilter2 ("TransmittedLightFilter2");
const std::string InternalShutter ("InternalShutter");
const std::string ReflectedLightShutter ("ReflectedLightShutter");
const std::string CondenserFrontlens ("CondenserFrontlens");
const std::string ExcitationFilter ("ExcitationFilter");
const std::string TransmittedLightFieldstop ("TransmittedLightFieldstop");
const std::string ReflectedLightAperture ("ReflectedLightAperture");
const std::string CondenserNA ("CondenserNA");
const std::string LightPath ("LightPath");
const std::string HalogenLampOn ("HalogenLampOn");
const std::string HalogenLampMode ("HalogenLampMode");
const std::string HalogenLampVoltage ("HalogenLampVoltage");
const std::string FluoLampMode ("FluoLampMode");
const std::string FluoLampIntensity ("FluoLampIntensity");
const std::string LightmanagerEnabled ("LightmanagerEnabled");
const std::string FocusPosition ("FocusPosition");
const std::string ImageCoordinateU ("ImageCoordinateU");
//const std::string StageXPosition ("StageXPosition");
const std::string ImageCoordinateV ("ImageCoordinateV");
//const std::string StageYPosition ("StageYPosition");
const std::string MicroscopeName ("MicroscopeName");
const std::string ObjectiveMagnification ("ObjectiveMagnification");
const std::string ObjectiveNA ("ObjectiveNA");
const std::string MicroscopeIllumination ("MicroscopeIllumination");
const std::string ExtraShutter1 ("ExtraShutter1");
const std::string ExtraShutter2 ("ExtraShutter2");
const std::string ExtraShutter3 ("ExtraShutter3");
const std::string ExtraFilter1 ("ExtraFilter1");
const std::string ExtraFilter2 ("ExtraFilter2");
const std::string ParfocalCorrection ("ParfocalCorrection");
const std::string ExtraShutter4 ("ExtraShutter4");
const std::string ExtraShutter5 ("ExtraShutter5");
const std::string ExtraShutter6 ("ExtraShutter6");
const std::string ExtraFilter3 ("ExtraFilter3");
const std::string ExtraFilter4 ("ExtraFilter4");
const std::string ObjectivePosition ("ObjectivePosition");
const std::string ObjectiveContrastMethod ("ObjectiveContrastMethod");
const std::string ObjectiveImmersionType ("ObjectiveImmersionType");
const std::string ReflectorPosition ("ReflectorPosition");
const std::string TransmissionFilter1Position ("TransmissionFilter1Position");
const std::string TransmittedLightFilter1Position ("TransmittedLightFilter1Position");
const std::string TransmissionFilter2Position ("TransmissionFilter2Position");
const std::string TransmittedLightFilter2Position ("TransmittedLightFilter2Position");
const std::string ExcitationFilterPosition ("ExcitationFilterPosition");
const std::string LampMirrorPosition ("LampMirrorPosition");
const std::string ExtraFilter1Position ("ExtraFilter1Position");
const std::string ExtraFilter2Position ("ExtraFilter2Position");
const std::string ExtraFilter3Position ("ExtraFilter3Position");
const std::string ExtraFilter4Position ("ExtraFilter4Position");
const std::string LightmanagerMode ("LightmanagerMode");
const std::string HalogenLampCalibrated ("HalogenLampCalibrated");
const std::string CondenserNAGoSpeed ("CondenserNAGoSpeed");
const std::string TransmittedLightFieldstopGoSpeed ("TransmittedLightFieldstopGoSpeed");
const std::string OptovarGoSpeed ("OptovarGoSpeed");
const std::string FocusCalibrated ("FocusCalibrated");
const std::string FocusBasicPosition ("FocusBasicPosition");
const std::string FocusPower ("FocusPower");
const std::string FocusBacklash ("FocusBacklash");
const std::string FocusMeasurementOrigin ("FocusMeasurementOrigin");
const std::string FocusMeasurementDistance ("FocusMeasurementDistance");
const std::string FocusSpeed ("FocusSpeed");
const std::string FocusGoSpeed ("FocusGoSpeed");
const std::string FocusStepDistance ("FocusStepDistance");
const std::string FocusInitPosition ("FocusInitPosition");
const std::string StageCalibrated ("StageCalibrated");
const std::string StagePower ("StagePower");
const std::string StageXBacklash ("StageXBacklash");
const std::string StageYBacklash ("StageYBacklash");
const std::string StageXSpeed ("StageXSpeed");
const std::string StageYSpeed ("StageYSpeed");
const std::string StageSpeed ("StageSpeed");
const std::string StageXGoSpeed ("StageXGoSpeed");
const std::string StageYGoSpeed ("StageYGoSpeed");
const std::string StageXStepDistance ("StageXStepDistance");
const std::string StageYStepDistance ("StageYStepDistance");
const std::string StageXInitPosition ("StageXInitPosition");
const std::string StageYInitPosition ("StageYInitPosition");
const std::string MicroscopeMagnification ("MicroscopeMagnification");
const std::string ReflectorMagnification ("ReflectorMagnification");
const std::string LampMirrorSelection ("LampMirrorSelection");
const std::string FocusDepth ("FocusDepth");
const std::string MicroscopeType ("MicroscopeType");
const std::string ObjectiveWorkingDistance ("ObjectiveWorkingDistance");
const std::string ReflectedLightApertureGoSpeed ("ReflectedLightApertureGoSpeed");
const std::string ExternalShutter ("ExternalShutter");
const std::string ObjectiveImmersionStopActive ("ObjectiveImmersionStopActive");
const std::string FocusStartSpeed ("FocusStartSpeed");
const std::string FocusAcceleration ("FocusAcceleration");
const std::string ReflectedLightFieldstop ("ReflectedLightFieldstop");
const std::string ReflectedLightFieldstopGoSpeed ("ReflectedLightFieldstopGoSpeed");
const std::string ReflectedLightFilter1 ("ReflectedLightFilter1");
const std::string ReflectedLightFilter2 ("ReflectedLightFilter2");
const std::string ReflectedLightFilter1Position ("ReflectedLightFilter1Position");
const std::string ReflectedLightFilter2Position ("ReflectedLightFilter2Position");
const std::string TransmittedLightAttenuator ("TransmittedLightAttenuator");
const std::string ReflectedLightAttenuator ("ReflectedLightAttenuator");
const std::string TransmittedLightShutter ("TransmittedLightShutter");
const std::string TransmittedLightAttenuatorGoSpeed ("TransmittedLightAttenuatorGoSpeed");
const std::string ReflectedLightAttenuatorGoSpeed ("ReflectedLightAttenuatorGoSpeed");
const std::string ApotomeFocusPosition ("ApotomeFocusPosition");
const std::string ApotomeFocusGoSpeed ("ApotomeFocusGoSpeed");
const std::string ApotomePhasePosition ("ApotomePhasePosition");
const std::string ApotomeSliderPosition ("ApotomeSliderPosition");
const std::string ApotomeGridLinesPerMillimeter ("ApotomeGridLinesPerMillimeter");
const std::string ApotomeWobbleMinPosition ("ApotomeWobbleMinPosition");
const std::string ApotomeWobbleMaxPosition ("ApotomeWobbleMaxPosition");
const std::string ApotomeWobbleSpeed ("ApotomeWobbleSpeed");
const std::string TransmittedLightVirtualFilterPosition ("TransmittedLightVirtualFilterPosition");
const std::string TransmittedLightVirtualFilter ("TransmittedLightVirtualFilter");
const std::string ReflectedLightVirtualFilterPosition ("ReflectedLightVirtualFilterPosition");
const std::string ReflectedLightVirtualFilter ("ReflectedLightVirtualFilter");
const std::string ReflectedLightHALMode ("ReflectedLightHALMode");
const std::string ReflectedLightHALVoltage ("ReflectedLightHALVoltage");
const std::string ReflectedLightHALColourTemperature ("ReflectedLightHALColourTemperature");
const std::string ContrastmanagerMode ("ContrastmanagerMode");
const std::string DazzleProtection ("DazzleProtection");
const std::string StageXCalibrated ("StageXCalibrated");
const std::string StageYCalibrated ("StageYCalibrated");
const std::string StageXPower ("StageXPower");
const std::string StageYPower ("StageYPower");
const std::string StageXStartSpeed ("StageXStartSpeed");
const std::string StageYStartSpeed ("StageYStartSpeed");
const std::string StageXAcceleration ("StageXAcceleration");
const std::string StageYAcceleration ("StageYAcceleration");
const std::string StageStartSpeed ("StageStartSpeed");
const std::string StageAcceleration ("StageAcceleration");
const std::string Zoom ("Zoom");
const std::string ZoomGoSpeed ("ZoomGoSpeed");
const std::string LightZoom ("LightZoom");
const std::string LightZoomGoSpeed ("LightZoomGoSpeed");
const std::string LightZoomCoupled ("LightZoomCoupled");
const std::string TransmittedLightHALMode ("TransmittedLightHALMode");
const std::string TransmittedLightHALVoltage ("TransmittedLightHALVoltage");
const std::string TransmittedLightHALColourTemperature ("TransmittedLightHALColourTemperature");
const std::string ReflectedLightCLMode ("ReflectedLightCLMode");
const std::string ReflectedLightCLIntensity ("ReflectedLightCLIntensity");
const std::string ReflectedLightCLColourTemperature ("ReflectedLightCLColourTemperature");
const std::string TransmittedLightCLMode ("TransmittedLightCLMode");
const std::string TransmittedLightCLIntensity ("TransmittedLightCLIntensity");
const std::string TransmittedLightCLColourTemperature ("TransmittedLightCLColourTemperature");
const std::string InfinitySpaceChangerPosition ("InfinitySpaceChangerPosition");
const std::string SidePortPercent ("SidePortPercent");
const std::string TwoTvVisCamChangerPosition ("TwoTvVisCamChangerPosition");
const std::string OcularPercent ("OcularPercent");
const std::string TwoTvCamerasChangerPosition ("TwoTvCamerasChangerPosition");
const std::string TopPortPercent ("TopPortPercent");
const std::string OcularShutter ("OcularShutter");
const std::string TwoTvCamerasChangerCube ("TwoTvCamerasChangerCube");
const std::string LightWaveLength ("LightWaveLength");
const std::string OcularMagnification ("OcularMagnification");
const std::string CameraAdapterMagnification ("CameraAdapterMagnification");
const std::string MicroscopePort ("MicroscopePort");
const std::string OcularTotalMagnification ("OcularTotalMagnification");
const std::string FieldOfView ("FieldOfView");
const std::string Ocular ("Ocular");
const std::string CameraAdapter ("CameraAdapter");
const std::string StageJoystickEnabled ("StageJoystickEnabled");
const std::string ContrastmanagerContrastMethod ("ContrastmanagerContrastMethod");
const std::string StageXJoystickEnabled ("StageXJoystickEnabled");
const std::string StageYJoystickEnabled ("StageYJoystickEnabled");
const std::string CameraFrameLeft ("CameraFrameLeft");
const std::string CameraFrameTop ("CameraFrameTop");
const std::string CameraFrameWidth ("CameraFrameWidth");
const std::string CameraFrameHeight ("CameraFrameHeight");
const std::string CameraFrameBinning ("CameraFrameBinning");
const std::string CameraFrameFull ("CameraFrameFull");
const std::string CameraFramePixelDistance ("CameraFramePixelDistance");
const std::string DataFormatUseScaling ("DataFormatUseScaling");
const std::string CameraFrameImageOrientation ("CameraFrameImageOrientation");
const std::string MeteorChannelInput ("MeteorChannelInput");
const std::string MeteorChannelSync ("MeteorChannelSync");
const std::string CameraWhiteBalanceRed ("CameraWhiteBalanceRed");
const std::string CameraWhiteBalanceGreen ("CameraWhiteBalanceGreen");
const std::string CameraWhiteBalanceBlue ("CameraWhiteBalanceBlue");
const std::string CameraFrameScalingFactor ("CameraFrameScalingFactor");
const std::string MeteorCameraType ("MeteorCameraType");
const std::string CameraExposureTimeValue ("CameraExposureTimeValue");
const std::string CameraExposureTimeAutoCalculate ("CameraExposureTimeAutoCalculate");
const std::string MeteorGainValue ("MeteorGainValue");
const std::string MeteorGainAutomatic ("MeteorGainAutomatic");
const std::string MeteorAdjustHue ("MeteorAdjustHue");
const std::string MeteorAdjustSaturation ("MeteorAdjustSaturation");
const std::string MeteorAdjustRedLow ("MeteorAdjustRedLow");
const std::string MeteorAdjustGreenLow ("MeteorAdjustGreenLow");
const std::string MeteorAdjustBlueLow ("MeteorAdjustBlueLow");
const std::string MeteorAdjustRedHigh ("MeteorAdjustRedHigh");
const std::string MeteorAdjustGreenHigh ("MeteorAdjustGreenHigh");
const std::string MeteorAdjustBlueHigh ("MeteorAdjustBlueHigh");
const std::string MeteorAdjustWhiteRefEnable ("MeteorAdjustWhiteRefEnable");
const std::string CameraExposureTimeCalculationControl ("CameraExposureTimeCalculationControl");
const std::string AxioCamFadingCorrectionEnable ("AxioCamFadingCorrectionEnable");
const std::string AxioCamSuspendLiveTimeout ("AxioCamSuspendLiveTimeout");
const std::string CameraLiveImage ("CameraLiveImage");
const std::string CameraLiveEnabled ("CameraLiveEnabled");
const std::string LiveImageSyncObjectName ("LiveImageSyncObjectName");
const std::string CameraLiveSpeed ("CameraLiveSpeed");
const std::string CameraImage ("CameraImage");
const std::string CameraImageWidth ("CameraImageWidth");
const std::string CameraImageHeight ("CameraImageHeight");
const std::string CameraImagePixelType ("CameraImagePixelType");
const std::string CameraImageShMemoryName ("CameraImageShMemoryName");
const std::string CameraLiveImageWidth ("CameraLiveImageWidth");
const std::string CameraLiveImageHeight ("CameraLiveImageHeight");
const std::string CameraLiveImagePixelType ("CameraLiveImagePixelType");
const std::string CameraLiveImageShMemoryName ("CameraLiveImageShMemoryName");
const std::string CameraLiveMaximumSpeed ("CameraLiveMaximumSpeed");
const std::string CameraLiveBinning ("CameraLiveBinning");
const std::string CameraLiveGainValue ("CameraLiveGainValue");
const std::string CameraLiveExposureTimeValue ("CameraLiveExposureTimeValue");
const std::string CameraLiveScalingFactor ("CameraLiveScalingFactor");
const std::string CameraCoAcqFastMode ("CameraCoAcqFastMode");
const std::string ImageIndexU ("ImageIndexU");
const std::string ImageIndexV ("ImageIndexV");
const std::string ImageIndexZ ("ImageIndexZ");
const std::string ImageIndexC ("ImageIndexC");
const std::string ImageIndexT ("ImageIndexT");
const std::string ImageIndexP ("ImageIndexP");
const std::string ImageIndexA ("ImageIndexA");
const std::string ImageCountP ("ImageCountP");
const std::string ImageCountA ("ImageCountA");
const std::string ImageIndexS ("ImageIndexS");
const std::string ImageIndexRaw ("ImageIndexRaw");
const std::string ImageCountZ ("ImageCountZ");
const std::string ImageCountC ("ImageCountC");
const std::string ImageCountT ("ImageCountT");
const std::string ImageCountU ("ImageCountU");
const std::string ImageCountV ("ImageCountV");
const std::string ImageCountS ("ImageCountS");
const std::string ImageOrgCoordinateU ("ImageOrgCoordinateU");
const std::string ImageOrgCoordinateV ("ImageOrgCoordinateV");
const std::string LayerDrawFlags ("LayerDrawFlags");
const std::string JobGroup ("JobGroup");
const std::string JobName ("JobName");
const std::string JobInfo ("JobInfo");
const std::string JobProgress ("JobProgress");
const std::string JobStatus ("JobStatus");
const std::string JobId ("JobId");
const std::string JobRemainingTime ("JobRemainingTime");

const std::string RelFocusPosition1 ("RelFocusPosition1");
const std::string RelFocusPosition2 ("RelFocusPosition2");
const std::string RelFocusPosition ("RelFocusPosition");
const std::string SidePortTurretposition ("SidePortTurretposition");
const std::string SidePortLeftPercent ("SidePortLeftPercent");
const std::string SidePortRightPercent ("SidePortRightPercent");
const std::string BasePortSliderPortPercent ("BasePortSliderPortPercent");
const std::string SidePortLeftFunction ("SidePortLeftFunction");
const std::string SidePortRightFunction ("SidePortRightFunction");
const std::string BasePortSliderPortFunction ("BasePortSliderPortFunction");
const std::string SidePortLightPath ("SidePortLightPath");
const std::string SidePortLightPathID ("SidePortLightPathID");
const std::string ObjectiveID ("ObjectiveID");
const std::string ReflectorID ("ReflectorID");


}
}
#endif /* ZEISSTAGMAPPING_H_ */
